/****************************************************
 * Billabong Dumb Computer implementation
 *
 * This code is intended only as an example of
 * how to implement an AI game playing client
 * that will interface with the game manager.
 *
 * This particular AI simply selects a random
 * walking move.  It does not generate any 
 * jumping moves.
 *
 * Feel free to use any code from this file.
 *
 * Kenrick Mock 9/26/03
 *
 ****************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "globals.h"
#include "move.h"
#include "movelist.h"
#include "client_ai.h"

/*********************************************
 * ComputeMove
 *
 * Generate a list of WALKING moves only
 * and then randomly picks one, turns it into a
 * string, and returns it.
 *
 * This is where you would put your minimax code
 * and search for the best move to make.
 *
 *   GenerateMoves and MoveToString is defined
 *   in the file move.c
 *
 **********************************************/
void ComputeMove(char board[XDIM][YDIM], piece_info pieces[], 
		char whoseturn, char *sMove)
{
 movelist mlist;
 int i;

 GenerateMoves(board, pieces, whoseturn, &mlist);
 i = random() % GetNumMoves(mlist);		/* Pick random move */
 MoveToString(&(mlist.mval[i]),sMove);		/* Convert to string */
}


/*********************************************
 * PlacePiece
 *
 * This routine figures out where to place our
 * pieces.  It just deterministically puts pieces
 * in the same slots unless one already exists there.
 * It checks the square from 9,9 to 11,11 and
 * tries 10 spots to guarantee placement if the
 * opponent puts pieces in the same place.
 *
 * The coordinates of where to place the piece
 * are returned in x and y.
 *
 *********************************************/
void PlacePiece(char board[XDIM][YDIM], int *x, int *y)
{
 int i;
 for (i=9; i<=11; i++) {
   if (board[i][9]==EMPTY) {
	*x=i; *y=9; return;
   }
 }
 for (i=9; i<=11; i++) {
   if (board[i][10]==EMPTY) {
	*x=i; *y=10; return;
   }
 }
 for (i=9; i<=11; i++) {
   if (board[i][11]==EMPTY) {
	*x=i; *y=11; return;
   }
 }
 if (board[12][9]==EMPTY) {
	*x=12; *y=9; return;
 }
 return;
}
 

