/******************************************************************
 * 9/26/03 KJM
 *
 * Dumb Billabong player.  This program is intended as an example
 * of how to interface with the game manager.  Feel free to use
 * any code from here as you see fit.
 *
 * For the following routines, you will have to modify this
 * code to initialize whatever structures you are using to
 * represent the game board, piece placement, etc.
 *
 *******************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include "globals.h"
#include "board.h"
#include "move.h"
#include "movelist.h"
#include "client_io.h"
#include "client_ai.h"

/*************************************************************
 *
 * The format we are expecting arguments is:
 *
 * argv[0] = program name   (ignore)
 * argv[1] = r or g
 *           If r, this means we are Red and should move first.
 *           If g, this means we are Green and should move second.
 *
 *************************************************************/
void ShowUsage(char *s);

main(int argc, char *argv[])
{
 char board[XDIM][YDIM];			/* Main game board */
 piece_info pieces[10];				/* Info on each piece */
 int i;	
 char ourcolor;
 int x,y;					/* The Move */
 char s[200];

 /* Initialize random number generator */
 srandom(time(NULL));

 /* Ensure we have two arguments */
 if (argc!=2) { 
   ShowUsage(argv[0]);
   exit(0);
 }

 /* First extract what color we are and who moves first.  */
 ourcolor = argv[1][0];		/* Should be either r or g */

 /* The following initializes some internal structures to 
  * play the game.  This program needs it, but yours
  * may not if your board data structures are set up differently.
 */
 InitBoard(board,pieces);

 /******* Piece placing phase *****/
 for (i=0; i<5; i++) 
 {
	/* If we're red then pick and send move */
	if (ourcolor=='r') {
	   PlacePiece(board,&x,&y);
	   /* Send position to opponent */
	   SendPlacingToOpponent(x,y); 
	}
	else 	/* Wait for opponent to pick position */
	{
	   GetPlacingFromOpponent(&x,&y);
	}
	board[x][y]=i;
 	pieces[i].curx=x; 
	pieces[i].cury=y;
	/* If we're green then pick and send move */
	if (ourcolor=='g') {
	   PlacePiece(board,&x,&y);
	   /* Send position to opponent */
	   SendPlacingToOpponent(x,y); 
	}
	else 	/* Wait for opponent to pick position */
	{
	   GetPlacingFromOpponent(&x,&y);
	}
	board[x][y]=i+5;
 	pieces[i+5].curx=x; 
	pieces[i+5].cury=y;
 }


 /*************************** MAIN PROGRAM LOOP *******************/
  
 /* Loop forever, getting a move and then sending our reply move. */

 while(1) {
    /* Handle red's turn */
    if (ourcolor=='r') {
	ComputeMove(board,pieces,'r',s);
	SendMoveToOpponent(s);
    }
    else {
    	GetMoveFromOpponent(s);
    }
    ApplyMove(board,pieces,s);
    if (GameOver(pieces,0)) exit(0);
    /* Handle green's turn */
    if (ourcolor=='g') {
	ComputeMove(board,pieces,'g',s);
	SendMoveToOpponent(s);
    }
    else {
    	GetMoveFromOpponent(s);
    }
    ApplyMove(board,pieces,s);
    if (GameOver(pieces,0)) exit(0);
 }
}

/****************************************************
 * ShowUsage
 *
 * Display proper parameters to invoke this program
 *
 ****************************************************/
void ShowUsage(char *s)
{
   printf("Usage: %s r|g\n",s);
   printf("e.g.: '%s r' for this program play red.\n",s);
   printf("e.g.: '%s g' for this program play green.\n",s);
   printf("Intended for usage only from the game manager program.\n");
}

