#ifndef __globals_h__
#define __globals_h__


#define MAXMOVES 300    /* If a piece exceeds this then the game is screwed */
			/* But there are only 216 squares on the board, so */
			/* it is highly unlikely to surpass this value */
			/* unless the game is stuck in a loop */

#define XDIM 17         /* Board dimension */
                        /* One larger than actual needed so  we don't have
			/* to map to a zero-based array */
#define YDIM 15

/* Board pieces values, 0-4 = red, 5-9 = green */
#define EMPTY -1
#define WATER -2

#define PRE_START 0	/* Hasn't passed start yet */
#define POST_START 1	/* Passed start */
#define BILLABONG 2	/* In the billabong, i.e. done with the lap */

/***********************************************************
 * The structure below stores information about
 * each piece:
 *    status = PRE_START, POST_START, or BILLABONG
 *    curx, cury = position on board
 *    nummoves = number of moves entered so far
 *    x[], y[] = arrays containing the coordinates of each move made
 *
 *    The coordinates are saved so we can tell if the piece has
 *    made a lap around the billabong.
 ***********************************************************/
struct piece_info_struct
{
	int status;	/* pre/post start */
	int curx, cury;
	int nummoves;   /* list of moves since crossing start
			   to determine if finished */
	int x[MAXMOVES];
	int y[MAXMOVES];
};
typedef struct piece_info_struct piece_info;

/***********************************************************
 * The move_piece structure is used to store information about
 * a move.  This is a move given in the format:
 *    W SRC_X SRC_Y DEST_X DEST_Y
 *    J SRC_X SRC_Y PIVOT1_X PIVO1_Y ... etc
 *    
 *    There is a routine that will parse from the string into
 *    this struct:
 *
 *    type = W or J, for Walk or Jump
 *    oldx, oldy = source coordinates of piece to move
 *    newx, newy = dest coordinates if walking one square
 *    numjumps = number pieces we are jumping over if a jumping move
 *    jumpx, jumpy = coordinates of pivot pieces.  Only supports
 *       up to 40 pivot pieces to jump.
 *
 *********************************************************/
#define MAXPIVOTS 40
typedef struct move_piece *Mptr;
struct move_piece {
	char type;
        int oldx;
        int oldy;
        int newx;
        int newy;
	int numjumps;
	int jumpx[MAXPIVOTS];
	int jumpy[MAXPIVOTS];
       };

typedef struct move_piece move_struct;

#define MOVELISTMAX 500

typedef struct move_piece mlist[MOVELISTMAX];
struct movecomb
 {
   mlist mval;
   int nummoves;
 };

/* Declare movelists to be of type movelist */
typedef struct movecomb movelist;

/* Heuristic Board Values */

int hboard[XDIM][YDIM];

#endif
