
/***************************************************************
 ************************** STOP !! ****************************
 ***************************************************************/

/*
   If you are looking at this file, it should only be for
   educational purposes.  If you want a code base to start
   from to use in your own program, use the CLIENT source
   code, not this code.  This code implements a server that
   is essentially just a pipe between two game programs.
 */

/* Name : gamemanager.c
 *
 * Revision 1.3 for Billabong 2003/9/26 by kjm
 *
 * Notes:  Based upon quixo.cc manager by idr
 * Revision 1.2  1996/10/08 19:47:12  idr
 * Revision 1.1  1996/10/07 22:24:59  idr
 * Initial revision
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "globals.h"
#include "board.h"
#include "move.h"

#define PLAYERHUMAN 'h'
#define PLAYER1 'r'
#define PLAYER2 'g'

void ShowUsage( const char * name );
void StartProgram( const char * program, FILE * fd[], const char * flags);
void StripNewlines(char *str);

/*****************************************************************************/
/* Name : main()
 *
 * Notes:
 *
 */

int main( int argc, char ** argv )
{
    char player1, player2;
    FILE *fd_player1[2], *fd_player2[2];
    int gameover=0;
    int whoseturn='r';
    int turn=1;
    char board[XDIM][YDIM];
    piece_info pieces[10];
    char s[200];
    int i;
    int x,y;

    printf("BILLABONG GAME MANGER v0.2\n\n");
    InitBoard(board,pieces);
    if ( argc < 3 )
    {
        ShowUsage( argv[ 0 ] );
        exit( 1 );
    }

    /* Create the structures for talking to our two players. */
    if (!strcmp("-",argv[1])) {
	player1 = PLAYERHUMAN;
    }
    else {
        player1 = PLAYER1;
    	StartProgram(argv[1], fd_player1, "r");
    }
    if (!strcmp("-",argv[2])) {
	player2 = PLAYERHUMAN;
    }
    else {
        player2 = PLAYER2;
    	StartProgram(argv[2], fd_player2, "g");
    }

    whoseturn='r';
    
    /* Read initial coordinates from players */
    for (i=0; i<=4; i++) {
	/* Get Red's Move */
	if (player1==PLAYERHUMAN) {
        	fprintf(stdout,"Red, enter coordinate for piece %d.\n",i+1);
		fscanf(stdin,"%d,%d",&x,&y);
    		fgets(s,10,stdin);
 	}
	else {
        	fprintf(stdout,
		  "Waiting for computer player red to place piece %d.\n",i+1);
		fscanf(fd_player1[0],"%d,%d",&x,&y);
    		fgets(s,10,fd_player1[0]);
	}
	fprintf(stdout,"Red placing piece %d at %d,%d\n",i+1,x,y);
	if (IsValidPlace(board,x,y)) {
		board[x][y]=i;
		pieces[i].curx=x;
		pieces[i].cury=y;
	}
	else {
		fprintf(stdout,
		   "Invalid place by red, to %d,%d.  Green wins!\n",x,y);
		exit(0);
	}
	ShowBoard(board,pieces);
	/* Get Green's Move */
	if (player2==PLAYERHUMAN) {
        	fprintf(stdout,"Green, enter coordinate for piece %d.\n",i+1);
		fscanf(stdin,"%d,%d",&x,&y);
    		fgets(s,10,stdin);
 	}
	else {
		/* Tell player 2 what player 1 did */
		fprintf(fd_player2[1],"%d,%d\n",x,y);
       		fflush(fd_player2[1]);
		/* Get player 2's move */
        	fprintf(stdout,
		  "Waiting for computer player green to place piece.\n");
		fscanf(fd_player2[0],"%d,%d",&x, &y);
    		fgets(s,10,fd_player2[0]);
	}
	fprintf(stdout,"Green placing piece %d at %d,%d\n",i+1,x,y);
	if (IsValidPlace(board,x,y)) {
		board[x][y]=i+5;
		pieces[i+5].curx=x;
		pieces[i+5].cury=y;
	}
	else {
		fprintf(stdout,
		   "Invalid place by green, to %d,%d.  Red wins!\n",x,y);
		exit(0);
	}
	ShowBoard(board,pieces);
	/* Tell player 1 what player 2 did. */
	if (player1!=PLAYERHUMAN) {
		fprintf(fd_player1[1],"%d,%d\n",x,y);
       		fflush(fd_player1[1]);
	}
    } 

    /* Play the game until somebody wins, or until they hit ^C. :) */
    while (!gameover)
    {
	ShowBoard(board,pieces);

        /* Get the first player's move and do it.  If the move is not valid,
         * then display the move to the "console" and break out with player 2
         * as the winner.
         */
	if (player1==PLAYERHUMAN) {
        	fprintf(stdout,"Red player, enter your move.\n");
		fgets(s,200,stdin);
 	}
	else {
        	fprintf(stdout,"Waiting for red computer player's move.\n");
		sprintf(s,"");
		fgets(s,200,fd_player1[0]);
	}
	if (s[0]=='j') s[0]='J';
	if (s[0]=='w') s[0]='W';
	fprintf(stdout,"Turn %d. Red moved : %s\n", turn, s);
	if (IsValid(board,pieces,whoseturn,s)) {
		ApplyMove(board,pieces,s);
  	}
    	else {
		/* Invalid move, a move was possible.  Player 2 wins */
		gameover=2;
		fprintf(stdout,"Red Player entered an invalid move! (%s)\n",
			s);
		goto EndOfGame;
    	}
	turn++;
	ShowBoard(board,pieces);
        /* Tell player 2 what player 1 did. */
	if (player2!=PLAYERHUMAN) {
		StripNewlines(s);
		fprintf(fd_player2[1],"%s\n",s);
       		fflush(fd_player2[1]);
	}
	/* Check if this was a game winner */
	gameover=GameOver(pieces,0);
	if (gameover) goto EndOfGame;
	whoseturn='g';

	/* Get player 2's move */
	if (player2!=PLAYERHUMAN) {
        	fprintf(stdout,"Waiting for green computer player's move.\n");
		fgets(s,200,fd_player2[0]);
 	}
	else {
        	fprintf(stdout,"Green player, enter your move.\n");
		sprintf(s,"");
		fgets(s,200,stdin);
	}
	if (s[0]=='j') s[0]='J';
	if (s[0]=='w') s[0]='W';
	fprintf(stdout,"Turn %d.  Green moved : %s\n", turn, s);
	if (IsValid(board,pieces,whoseturn,s)) {
		ApplyMove(board,pieces,s);
     	}
	else {
		/* Invalid move, a move was possible.  Player 1 wins */
		gameover=1;
		fprintf(stdout,"Green entered an invalid move! (%s)\n",
			s);
		goto EndOfGame;
    	}
        /* Tell player 1 what player 2 did. */
	if (player1!=PLAYERHUMAN) {
		StripNewlines(s);
		fprintf(fd_player1[1],"%s\n", s);
       		fflush(fd_player1[1]);
	}

	gameover=GameOver(pieces,0);
	if (gameover) goto EndOfGame;
	whoseturn='r';
	turn++;
    }

EndOfGame:

    /* Tell them who won, close up shop, and terminate. */
    ShowBoard(board,pieces);
    printf("\nGAME OVER.\n");
    if (gameover==1) fprintf(stdout,"RED Wins!\n");
    else fprintf(stdout,"GREEN Wins!\n");
    return( 0 );
}


/*****************************************************************************/
/* Name : StartProgram()
 *
 * Notes: This function starts a new process and runs the named program.  It
 *        fills fd[0] with a file handle that is tied to the new program's
 *        STDOUT (i.e., you can read the output of the child program through
 *        this file), and fills fd[1] with a file handle that is tied to the
 *        new program's STDIN.
 */

void StartProgram( const char * program, FILE* fd[], const char * player )
{
    int     pipeIn[ 2 ];        /* Provides STDIN for the CHILD.    */
    int     pipeOut[ 2 ];       /* Provides STDOUT for the CHILD.   */

    /* First, create the two pipes that we will use to communicate with the
     * child processes.
     */

    if ( pipe( pipeIn ) == -1 )
    {
        perror( "pipe( pipeIn )" );
        exit( 1 );
    }

    if ( pipe( pipeOut ) == -1 )
    {
        perror( "pipe( pipeOut )" );
        exit( 1 );
    }

    /* Create the child process! */
    if ( fork() == 0 )
    {
        /* Close the default STDIN and STDOUT file handles.  We want our
         * STDIN and STDOUT tied directly to our parent process.
         */

        close( STDIN_FILENO );
        close( STDOUT_FILENO );


        /* Close the write part of our STDIN pipe and the read part of our
         * STDOUT pipe.  As the child, we can't use these.
         */

        close( pipeIn[ 1 ] );
        close( pipeOut[ 0 ] );



        if ( dup2( pipeIn[ 0 ], STDIN_FILENO ) == -1 )
        {
            perror( "dup2( pipeIn[ 0 ], STDIN_FILENO )" );
            exit( 1 );
        }

        if ( dup2( pipeOut[ 1 ], STDOUT_FILENO ) == -1 )
        {
            perror( "dup2( pipeOut[ 1 ], STDOUT_FILENO )" );
            exit( 1 );
        }


        /* Run the specified program. */

        execl( program, program, player, NULL );


        /* If we get to this point, it means that execl() failed.  Let's
         * print an error message and exit.
         */

        perror( "execl()" );
        exit( 1 );
    }

    /* Close the read part of the child's STDIN pipe and the out part of
     * the child's STDOUT pipe.  As the parent, we (can't) won't be using
     * these parts.
     */

    close( pipeIn[ 0 ] );
    close( pipeOut[ 1 ] );

    /* Convert the parts of the pipes that we will be using to normal
     * stdio style FILEs.
     */

    fd[ 0 ] = fdopen( pipeOut[ 0 ], "r" );
    if ( fd[ 0 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }

    fd[ 1 ] = fdopen( pipeIn[ 1 ], "w" );
    if ( fd[ 1 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }
}




/*****************************************************************************/
/* Name : ShowUsage()
 *
 * Notes: This function attempts to show the user how to properly run our
 *        little program.
 */

void ShowUsage( const char * name )
{
    fprintf( stderr, "\nUsage: %s red_program green_program \n" \
                     "If the player_program is \"-\", then the player " \
                     "actually a human player.\n",
             name );
}

/*****************************************************************************/
/* Name : StripNewLines
 *
 * Notes: This function removes newlines from the string, replacing them
 * with the \0 character.  Keyboard input may leave newlines at the end
 * so remove them with this function.
 */

void StripNewlines(char *str)
{
    int i;
    for (i=0; i<strlen(str); i++) {
	if (str[i]=='\n') {
		str[i]='\0';
		return;
	}
    }
}

