/**************************************************
 * movelist.c
 *
 * This code implements a data structure that
 * stores moves.  It uses an array up to MOVELISTMAX
 * moves (see definition in globals.h)
 *
 ***************************************************/
#include <stdio.h>
#include "globals.h"

/****************************************************
 * Clear move list, internal function
 ***************************************************/
int ClearMoveList(movelist *m)
{
 m->nummoves=0;
}

/****************************************************
 * Add a move to the move list.
 * Uses memcpy to copy the arrays for speed.
 ***************************************************/
int AddToMoveList(movelist *m, move_struct *move)
{
 int moveindex=m->nummoves;

 if (moveindex >= MOVELISTMAX) {
	 fprintf(stderr,"Exceeded move list size!!\n");
	 return 0;
 }
 (m->mval[moveindex]).type=move->type;
 (m->mval[moveindex]).oldx=move->oldx;
 (m->mval[moveindex]).oldy=move->oldy;
 (m->mval[moveindex]).newx=move->newx;
 (m->mval[moveindex]).newy=move->newy;
 (m->mval[moveindex]).numjumps=move->numjumps;
 memcpy((void *) ((m->mval[moveindex]).jumpx), (const void *) (move->jumpx),
 	(size_t) 20*sizeof(int));
 memcpy((void *) ((m->mval[moveindex]).jumpy), (const void *) (move->jumpy), 
 	(size_t) 20*sizeof(int));
 m->nummoves++;
 return 1;
}

/****************************************************
 * Return number of moves entered in the movelist (the length) 
 ***************************************************/
int GetNumMoves(movelist m)
{
 return m.nummoves;
}

